/*
 * Copyright (c) 2004, Nate Nielsen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *     * Redistributions of source code must retain the above 
 *       copyright notice, this list of conditions and the 
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the 
 *       above copyright notice, this list of conditions and 
 *       the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be 
 *       used to endorse or promote products derived from this 
 *       software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 * 
 *
 * CONTRIBUTORS
 *  Nate Nielsen <nielsen@memberwebs.com>
 *
 */ 

#ifndef __RTFREADER_H__
#define __RTFREADER_H__

#include "usuals.h"
#include <stack>
#include <stdio.h>

class RtfReader;

class RtfHandler  
{
public:
	virtual void startDocument(RtfReader* reader) = 0;
	virtual void endDocument() = 0;
  virtual void controlWord(const string& cw, int flags, int param) = 0;
	virtual void groupStart() = 0; 
	virtual void groupEnd() = 0;
	virtual void charData(wstring data) = 0;

	static const int kAsterisk;
	static const int kHasParam;
	static const int kIsEncoded;
};

class RtfReader  
{
public:
	RtfReader();
	virtual ~RtfReader();


	bool parse(string fileName);
	bool parse(FILE* file);

	void setHandler(RtfHandler* handler)
		{ m_handler = handler; }
	string getParseErrors() const
		{ return m_parseErrors; }
	int getDepth() const
		{ return m_depth; }
	void setHexParse(bool parse)
		{ m_parseHex = parse; }
	void setUnicode(bool unicode);

protected:
	RtfHandler* m_handler;
	int m_depth;
	bool m_parseHex;
	string m_parseErrors;

	// Unicode handling
	bool m_parseUnicode;
	typedef std::stack<int> StackInt;
	StackInt m_uniEatStack;
	int m_uniEat;

private:

	struct RtfContext
	{
		FILE* file;
		bool isData;
		wstring data;
	};

	bool parseControlWord(RtfContext& cx);
	bool parseHexChar(RtfContext& cx, int num);
	void sendControlWord(RtfContext& cx, string cw, int flags, int param);
	void sendData(RtfContext& cx, wchar_t ch);
	void sendData(RtfContext& cx, wstring data);
	void emptyData(RtfContext& cx);
};

#endif // __RTFREADER_H__
