/*
 * Copyright (c) 2004, Nate Nielsen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *     * Redistributions of source code must retain the above 
 *       copyright notice, this list of conditions and the 
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the 
 *       above copyright notice, this list of conditions and 
 *       the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be 
 *       used to endorse or promote products derived from this 
 *       software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 * 
 *
 * CONTRIBUTORS
 *  Nate Nielsen <nielsen@memberwebs.com>
 *
 */ 

#include "usuals.h"
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include "rtfreader.h"
#include "rtfanalyser.h"

int usage()
{
	fprintf(stderr, "usage: rtfx <inrtf> <outxml>\n");
	return 2;
}

int main(int argc, char* argv[])
{
	if(argc < 3)
		return usage();

	try
	{
		FILE* file = fopen(argv[1], "rb");

		if(!file)
		{
			fprintf(stderr, "rtfx: couldn't open file: %s: %s\n", argv[1], strerror(errno));
			return 1;
		}

		RtfParserOptions options;
 
		RtfParser handler(options);
		RtfReader rtf;
		rtf.setHandler(&handler);
		bool ret = rtf.parse(file);
		fclose(file);

		if(!ret)
		{
			fprintf(stderr, "rtfx: rtf parse failed: %s\n", rtf.getParseErrors().c_str());
			return 1;
		}


		DOM::Document doc = handler.getDocument();
		string xml = doc.serialize();

		FILE* out = fopen(argv[2], "wb");
		if(!out)
		{
			fprintf(stderr, "rtfx: couldn't open file: %s: %s\n", argv[2], strerror(errno));
			return 1;
		}

		fwrite(xml.c_str(), 1, xml.length(), out);
		fclose(out);
		return 0;

	}
	catch(DOM::DOMException& e)
	{
		fprintf(stderr, "rtfx: xml dom error: %s\n", e.getMessage());
	}

	return 1;
}
